==> icoutils_0.13.0.orig.tar.gz: icoutils-0.13.0/wrestool/main.c <==
/* icoutils - A set of programs dealing with MS Windows icons and cursors.
 * Copyright (C) 1998-1999 Oskar Liljeblad
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "wrestool.h"
#define PROGRAM "wrestool"

/*
 * Exported variables
 */
char *prgname;
char *arg_output;
int arg_verbosity;
boolean arg_raw;
FILE *verbose_file;

/*
 * Static variables
 */
static char *arg_type;
static char *arg_name;
static char *arg_language;
static int arg_action;
static char *res_types[] = {
	/* 0x01: */
	"cursor", "bitmap", "icon", "menu", "dialog", "string",
	"fontdir", "font", "accelerator", "rcdata", "messagelist",
	"group_cursor", NULL, "group_icon", NULL,
	/* the following are not defined in winbase.h, but found in wrc. */
	/* 0x10: */ 
	"version", "dlginclude", NULL, "plugplay", "vxd",
	"anicursor", "aniicon"
};
#define RES_TYPE_COUNT (sizeof(res_types)/sizeof(char *))

/*
 * Prototypes for static functions
 */
static char *res_type_string_to_id (char *);
static char *get_extract_extension (char *);

/* res_type_id_to_string:
 *   Translate a numeric resource type to it's corresponding string type.
 *   (For informative-ness.)
 */
char *
res_type_id_to_string (int id)
{
	if (id == 241)
		return "toolbar";
	if (id > 0 || id-1 < RES_TYPE_COUNT)
		return res_types[id-1];
	return NULL;
}

/* res_type_string_to_id:
 *   Translate a resource type string to integer.
 *   (Used to convert the --type option.)
 */
static char *
res_type_string_to_id (char *type)
{
	static char *res_type_ids[] = {
		"-1", "-2", "-3", "-4", "-5", "-6", "-7", "-8", "-9", "-10",
		"-11", "-12", NULL, "-14", NULL, "-16", "-17", NULL, "-19",
		"-20", "-21", "-22"
	};
	int c;

	if (type == NULL)
		return NULL;

	for (c = 0 ; c < RES_TYPE_COUNT ; c++) {
		if (res_types[c] != NULL && !strcasecmp(type, res_types[c]))
			return res_type_ids[c];
	}

	return type;
}

/* get_extract_extension:
 *   Return extension for files of a certain resource type
 *
 */
static char *
get_extract_extension (char *type)
{
	int value;

	type = res_type_string_to_id(type);
	STRIP_RES_ID_FORMAT(type);
	if (str2int(type, &value)) {
		if (value == 14)
			return ".ico";
		if (value == 12)
			return ".cur";
	}

	return "";
}

#define SET_IF_NULL(x,def) ((x) = ((x) == NULL ? (def) : (x)))

/* get_destination_name:
 *   Make a filename for a resource that is to be extracted.
 */
char *
get_destination_name (WinLibrary *fi, char *type, char *name, char *lang)
{
	static char filename[1024];

	/* initialize --type, --name and --language options */
	SET_IF_NULL(type, "");
	SET_IF_NULL(name, "");
	if (!strcmp(lang, "1033"))
		lang = NULL;
	STRIP_RES_ID_FORMAT(type);
	STRIP_RES_ID_FORMAT(name);
	STRIP_RES_ID_FORMAT(lang);

	/* returning NULL means that output should be done to stdout */

	/* if --output not specified, write to STDOUT */
	if (arg_output == NULL)
		return NULL;

	/* if --output'ing to a directory, make filename */
	if (is_directory(arg_output) || STR_LASTCHR(arg_output) == '/') {
		/* char *tmp = strdup(fi->name);
		if (tmp == NULL)
			malloc_failure(); */

		snprintf (filename, 1024, "%s%s%s_%s_%s%s%s%s",
				  arg_output,
			      (STR_LASTCHR(arg_output) != '/' ? "/" : ""),
				  basename(fi->name),
				  type,
				  name,
				  (lang != NULL && fi->is_PE_binary ? "_" : ""),
				  (lang != NULL && fi->is_PE_binary ? lang : ""),
				  get_extract_extension(type));
		/* free(tmp); */
		return filename;
	}

	/* otherwise, just return the --output argument */
	return arg_output;
}

#define HELP_MESSAGE "Usage: %s [OPTION]... [FILE]...\n\
\n\
Commands:\n\
  -x, --extract           extract resources\n\
  -l, --list              output list of resources (default)\n\
\n\
Filters:\n\
  -t, --type=[+|-]ID      resource type identifier\n\
  -n, --name=[+|-]ID      resource name identifier\n\
  -L, --language=[+|-]ID  resource language identifier\n\
      --all               perform operation on all resource (default)\n\
\n\
Miscellaneous:\n\
  -o, --output=PATH       where to place extracted files\n\
  -R, --raw               do not parse resource contents\n\
  -v, --verbose           explain what is being done\n\
      --help              display this help and exit\n\
      --version           output version information and exit\n\
\n\
A leading `+' in --type, name or language options indicates a true string\n\
identifier. Similarly, `-' indicates a true numeric identifier.\n\
\n\
Report bugs to " BUG_EMAIL ".\n"

int
main (int argc, char **argv)
{
	int c;

	arg_type = arg_name = arg_language = NULL;
	arg_verbosity = 0;
	arg_raw = FALSE;
	arg_action = ACTION_LIST;
	prgname = argv[0];

	/* analyse arguments */
	while (TRUE) {
		int option_index = 0;
		static struct option long_options[] = {
			{ "type",		required_argument,	NULL, 't' },
			{ "name", 		required_argument,	NULL, 'n' },
			{ "language",	required_argument,	NULL, 'L' },
			{ "output",     required_argument,  NULL, 'o' },
			{ "all",		no_argument,		NULL, 'a' },
			{ "raw",        no_argument,        NULL, 'R' },
			{ "extract",	no_argument,		NULL, 'x' },
			{ "list",		no_argument,		NULL, 'l' },
			{ "verbose",	no_argument,		NULL, 'v' },
			{ "version",	no_argument,		NULL, 150 },
			{ "help",		no_argument,		NULL, 151 },
			{ 0, 0, 0, 0 }
		};
		c = getopt_long (argc, argv, "t:n:L:o:aRrxlv", long_options, &option_index);
		if (c == EOF)
			break;

		switch (c) {
			case 't': arg_type = optarg; break;
			case 'n': arg_name = optarg; break;
			case 'L': arg_language = optarg; break;
			case 'a': arg_type = arg_name = arg_language = NULL; break;
			case 'R': arg_raw = TRUE; break;
			case 'x': arg_action = ACTION_EXTRACT; break;
			case 'l': arg_action = ACTION_LIST; break;
			case 'v': arg_verbosity++; break;
			case 'o': arg_output = optarg; break;
			case 150:
				printf("%s (%s) %s\n", PROGRAM, PACKAGE, VERSION);
				return 0;
			case 151:
				printf(HELP_MESSAGE, prgname);
				return 0;
			case '?':
			default:
				return 1;
		}
	}

	verbose_file = (arg_output == NULL) ? stderr : stdin;

	/* warn about unnecessary options */
	if (arg_action == ACTION_LIST) {
		if (arg_language != NULL && (arg_name == NULL || arg_type == NULL))
			warn_printf("--language has no effect without --name and --type\n");
		if (arg_name != NULL && arg_type == NULL)
			warn_printf("--name has no effect without --type");
	}

	/* translate --type option from resource type string to integer */
	arg_type = res_type_string_to_id(arg_type);

	/* make sure at least one file has been specified */
	if (optind >= argc) {
		P_GEN_ERROR("missing file argument");
		warn_printf("Try `%s --help' for more information.\n", prgname);
		return 1;
	}

	/* for each file */
	for (c = optind ; c < argc ; c++) {
		WinLibrary fi;
		
		/* initiate stuff */
		fi.file = NULL;
		fi.memory = NULL;

		/* get file size */
		fi.name = argv[c];
		if (!get_file_size(fi.name, &(fi.total_size))) {
			P_OBJ_SYS_ERROR(fi.name);
			goto cleanup;
		}
		if (fi.total_size == 0) {
			P_OBJ_ERROR(fi.name, "file has a size of 0");
			goto cleanup;
		}

		/* open file */
		fi.file = fopen(fi.name, "rb");
		if (fi.file == NULL) {
			P_OBJ_SYS_ERROR(fi.name);
			goto cleanup;
		}
		
		/* read all of file */
		fi.memory = malloc(fi.total_size);
		if (fi.memory == NULL)
			malloc_failure();
		if (fread(fi.memory, fi.total_size, 1, fi.file) != 1) {
			P_OBJ_SYS_ERROR(fi.name);
			goto cleanup;
		}

		/* identify file and find resource table */
		if (!read_library (&fi)) {
			/* error reported by read_library */
			goto cleanup;
		}

		verbose_printf("file is a %s\n",
			fi.is_PE_binary ? "Windows NT `PE' binary" : "Windows 3.1 `NE' binary");

		/* warn about more unnecessary options */
		if (!fi.is_PE_binary && arg_language != NULL)
			P_OBJ_WARN(fi.name, "--language has no effect because file is 16-bit binary");

		/* do the specified command */
		if (arg_action == ACTION_LIST) {
			do_resources (&fi, arg_type, arg_name, arg_language, print_resources_callback);
			/* errors will be printed by the callback */
		} else if (arg_action == ACTION_EXTRACT) {
			do_resources (&fi, arg_type, arg_name, arg_language, extract_resources_callback);
			/* errors will be printed by the callback */
		}

		/* free stuff and close file */
		cleanup:
		if (fi.file != NULL)
			fclose(fi.file);
		if (fi.memory != NULL)
			free(fi.memory);
	}

	return 0;
}

==> icoutils_0.22.0.orig.tar.gz: icoutils-0.22.0/wrestool/main.c <==
/* icoutils - A set of programs dealing with MS Windows icons and cursors.
 * Copyright (C) 1998-1999 Oskar Liljeblad
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <config.h>
#include "common/error.h"
#include "common/memory.h"
#include "common/intparse.h"
#include "common/io-utils.h"
#include "common/string-utils.h"
#include "wrestool.h"
#include "strcase.h"
#include "dirname.h"
#define PROGRAM "wrestool"

#ifndef HAVE_DECL_PROGRAM_INVOCATION_NAME
char *program_invocation_name;
#endif

/*
 * Exported variables
 */
char *prgname;
char *arg_output;
int arg_verbosity;
bool arg_raw;
FILE *verbose_file;

/*
 * Static variables
 */
static char *arg_type;
static char *arg_name;
static char *arg_language;
static int arg_action;
static char *res_types[] = {
	/* 0x01: */
	"cursor", "bitmap", "icon", "menu", "dialog", "string",
	"fontdir", "font", "accelerator", "rcdata", "messagelist",
	"group_cursor", NULL, "group_icon", NULL,
	/* the following are not defined in winbase.h, but found in wrc. */
	/* 0x10: */ 
	"version", "dlginclude", NULL, "plugplay", "vxd",
	"anicursor", "aniicon"
};
#define RES_TYPE_COUNT (sizeof(res_types)/sizeof(char *))

/*
 * Prototypes for static functions
 */
static char *res_type_string_to_id (char *);
static char *get_extract_extension (char *);

/* res_type_id_to_string:
 *   Translate a numeric resource type to it's corresponding string type.
 *   (For informative-ness.)
 */
char *
res_type_id_to_string (int id)
{
	if (id == 241)
		return "toolbar";
	if (id > 0 && id <= RES_TYPE_COUNT)
		return res_types[id-1];
	return NULL;
}

/* res_type_string_to_id:
 *   Translate a resource type string to integer.
 *   (Used to convert the --type option.)
 */
static char *
res_type_string_to_id (char *type)
{
	static char *res_type_ids[] = {
		"-1", "-2", "-3", "-4", "-5", "-6", "-7", "-8", "-9", "-10",
		"-11", "-12", NULL, "-14", NULL, "-16", "-17", NULL, "-19",
		"-20", "-21", "-22"
	};
	int c;

	if (type == NULL)
		return NULL;

	for (c = 0 ; c < RES_TYPE_COUNT ; c++) {
		if (res_types[c] != NULL && !strcasecmp(type, res_types[c]))
			return res_type_ids[c];
	}

	return type;
}

/* get_extract_extension:
 *   Return extension for files of a certain resource type
 *
 */
static char *
get_extract_extension (char *type)
{
	uint16_t value;

	type = res_type_string_to_id(type);
	STRIP_RES_ID_FORMAT(type);
	if (parse_uint16(type, &value)) {
		if (value == 14)
			return ".ico";
		if (value == 12)
			return ".cur";
	}

	return "";
}

#define SET_IF_NULL(x,def) ((x) = ((x) == NULL ? (def) : (x)))

/* get_destination_name:
 *   Make a filename for a resource that is to be extracted.
 */
char *
get_destination_name (WinLibrary *fi, char *type, char *name, char *lang)
{
	static char filename[1024];

	/* initialize --type, --name and --language options */
	SET_IF_NULL(type, "");
	SET_IF_NULL(name, "");
	if (!strcmp(lang, "1033"))
		lang = NULL;
	STRIP_RES_ID_FORMAT(type);
	STRIP_RES_ID_FORMAT(name);
	STRIP_RES_ID_FORMAT(lang);

	/* returning NULL means that output should be done to stdout */

	/* if --output not specified, write to STDOUT */
	if (arg_output == NULL)
		return NULL;

	/* if --output'ing to a directory, make filename */
	if (is_directory(arg_output) || ends_with(arg_output, "/")) {
		/* char *tmp = strdup(fi->name);
		if (tmp == NULL)
			malloc_failure(); */

		snprintf (filename, 1024, "%s%s%s_%s_%s%s%s%s",
				  arg_output,
			      (ends_with(arg_output, "/") ? "" : "/"),
				  base_name(fi->name),
				  type,
				  name,
				  (lang != NULL && fi->is_PE_binary ? "_" : ""),
				  (lang != NULL && fi->is_PE_binary ? lang : ""),
				  get_extract_extension(type));
		/* free(tmp); */
		return filename;
	}

	/* otherwise, just return the --output argument */
	return arg_output;
}

#define HELP_MESSAGE "Usage: %s [OPTION]... [FILE]...\n\
\n\
Commands:\n\
  -x, --extract           extract resources\n\
  -l, --list              output list of resources (default)\n\
\n\
Filters:\n\
  -t, --type=[+|-]ID      resource type identifier\n\
  -n, --name=[+|-]ID      resource name identifier\n\
  -L, --language=[+|-]ID  resource language identifier\n\
      --all               perform operation on all resource (default)\n\
\n\
Miscellaneous:\n\
  -o, --output=PATH       where to place extracted files\n\
  -R, --raw               do not parse resource contents\n\
  -v, --verbose           explain what is being done\n\
      --help              display this help and exit\n\
      --version           output version information and exit\n\
\n\
A leading `+' in --type, name or language options indicates a true string\n\
identifier. Similarly, `-' indicates a true numeric identifier.\n\
\n\
Report bugs to " BUG_EMAIL ".\n"

int
main (int argc, char **argv)
{
	int c;

	arg_type = arg_name = arg_language = NULL;
	arg_verbosity = 0;
	arg_raw = false;
	arg_action = ACTION_LIST;
	prgname = argv[0];

#ifndef HAVE_DECL_PROGRAM_INVOCATION_NAME
    	program_invocation_name = argv[0];
#endif

	/* analyse arguments */
	while (true) {
		int option_index = 0;
		static struct option long_options[] = {
			{ "type",		required_argument,	NULL, 't' },
			{ "name", 		required_argument,	NULL, 'n' },
			{ "language",	required_argument,	NULL, 'L' },
			{ "output",     required_argument,  NULL, 'o' },
			{ "all",		no_argument,		NULL, 'a' },
			{ "raw",        no_argument,        NULL, 'R' },
			{ "extract",	no_argument,		NULL, 'x' },
			{ "list",		no_argument,		NULL, 'l' },
			{ "verbose",	no_argument,		NULL, 'v' },
			{ "version",	no_argument,		NULL, 150 },
			{ "help",		no_argument,		NULL, 151 },
			{ 0, 0, 0, 0 }
		};
		c = getopt_long (argc, argv, "t:n:L:o:aRrxlv", long_options, &option_index);
		if (c == EOF)
			break;

		switch (c) {
			case 't': arg_type = optarg; break;
			case 'n': arg_name = optarg; break;
			case 'L': arg_language = optarg; break;
			case 'a': arg_type = arg_name = arg_language = NULL; break;
			case 'R': arg_raw = true; break;
			case 'x': arg_action = ACTION_EXTRACT; break;
			case 'l': arg_action = ACTION_LIST; break;
			case 'v': arg_verbosity++; break;
			case 'o': arg_output = optarg; break;
			case 150:
				printf("%s (%s) %s\n", PROGRAM, PACKAGE, VERSION);
				return 0;
			case 151:
				printf(HELP_MESSAGE, prgname);
				return 0;
			case '?':
			default:
				return 1;
		}
	}

	verbose_file = (arg_output == NULL) ? stderr : stdin;

	/* warn about unnecessary options */
	if (arg_action == ACTION_LIST) {
		if (arg_language != NULL && (arg_name == NULL || arg_type == NULL))
			warn("--language has no effect without --name and --type");
		if (arg_name != NULL && arg_type == NULL)
			warn("--name has no effect without --type");
	}

	/* translate --type option from resource type string to integer */
	arg_type = res_type_string_to_id(arg_type);

	/* make sure at least one file has been specified */
	if (optind >= argc) {
		warn("missing file argument\nTry `%s --help' for more information.", prgname);
		return 1;
	}

	/* for each file */
	for (c = optind ; c < argc ; c++) {
		WinLibrary fi;
		
		/* initiate stuff */
		fi.file = NULL;
		fi.memory = NULL;

		/* get file size */
		fi.name = argv[c];
		fi.total_size = file_size(fi.name);
		if (fi.total_size == -1) {
			die_errno("%s", fi.name);
			goto cleanup;
		}
		if (fi.total_size == 0) {
			warn("%s: file has a size of 0", fi.name);
			goto cleanup;
		}

		/* open file */
		fi.file = fopen(fi.name, "rb");
		if (fi.file == NULL) {
			die_errno("%s", fi.name);
			goto cleanup;
		}
		
		/* read all of file */
		fi.memory = xmalloc(fi.total_size);
		if (fread(fi.memory, fi.total_size, 1, fi.file) != 1) {
			die_errno("%s", fi.name);
			goto cleanup;
		}

		/* identify file and find resource table */
		if (!read_library (&fi)) {
			/* error reported by read_library */
			goto cleanup;
		}

	//	verbose_printf("file is a %s\n",
	//		fi.is_PE_binary ? "Windows NT `PE' binary" : "Windows 3.1 `NE' binary");

		/* warn about more unnecessary options */
		if (!fi.is_PE_binary && arg_language != NULL)
			warn("%s: --language has no effect because file is 16-bit binary", fi.name);

		/* do the specified command */
		if (arg_action == ACTION_LIST) {
			do_resources (&fi, arg_type, arg_name, arg_language, print_resources_callback);
			/* errors will be printed by the callback */
		} else if (arg_action == ACTION_EXTRACT) {
			do_resources (&fi, arg_type, arg_name, arg_language, extract_resources_callback);
			/* errors will be printed by the callback */
		}

		/* free stuff and close file */
		cleanup:
		if (fi.file != NULL)
			fclose(fi.file);
		if (fi.memory != NULL)
			free(fi.memory);
	}

	return 0;
}
